import pytest
import unittest
from unittest.mock import patch

from pathlib import Path
import json

from startwork.models.CreateProject import CreateProject
from startwork.models.SelectProject import SelectProject

prompt_mock_values = [
  {"name": "sample_name", "project_path": "/home"},
]

prompt_mock_selected_project_values = [
  {"selected_project": prompt_mock_values[0]["name"]}
]

class TestCreateProject(unittest.TestCase):
  # CLASS SETUP
  project_list_path = Path(__file__).parent / "projects_list.json"

  @pytest.fixture(autouse=True)
  def _capsys(self, capsys):
    self.capsys = capsys

  # TESTS
  def test_if_is_function(self):
    assert callable(SelectProject.run)

  @patch(
    "startwork.models.CreateProject.prompt",
    return_value=prompt_mock_values[0]
  )
  @patch(
    "startwork.models.SelectProject.prompt",
    return_value=prompt_mock_selected_project_values[0]
  )
  def test_happy_path(
    self,
    mock_inquirer_prompt_select,
    mock_inquirer_prompt_create
  ):
    # set json for the test
    CreateProject.run(self.project_list_path)
    with open(self.project_list_path, "r") as file:
      result = json.load(file)
      assert result == prompt_mock_values
    # clean output
    self.capsys.readouterr()

    # do the test it self
    expected_project = prompt_mock_values[0]
    selected_project = SelectProject.run(self.project_list_path)
    assert selected_project == expected_project

    # clean json for the next test 
    with open(self.project_list_path, "w") as outfile:
      outfile.write(json.dumps([], indent=4))

    with open(self.project_list_path, "r") as file:
      assert json.load(file) == []



import json
from inquirer import List, Confirm, prompt

from .GenericProjectActionsModel import GenericProjectActionsModel

class DeleteProject(GenericProjectActionsModel):
  @staticmethod
  def run(project_list_path):
    projects_list = []

    with open(project_list_path, 'r') as openfile:
      projects_list = json.load(openfile)
    
    projects_keys=[project["name"] for project in projects_list]

    questions = [
      List(
        'selected_project',
        message="Delete a project",
        choices=projects_keys,
      ),
      Confirm(
        'delete_confirmed',
        message="This project will be deleted. Continue?",
        default=False
      ),
      Confirm(
        'delete_confirmed2',
        message="ARE YOU SURE???",
        default=False
      )
    ]

    answers = prompt(questions)

    DeleteProject._validate_selected_project_name(answers["selected_project"])
    
    if not (answers['delete_confirmed'] and answers['delete_confirmed2']):
      raise Exception("Deletion Aborted!")

    projects_list = list(
      filter(
        lambda project: project['name'] != answers['selected_project'],
        projects_list
      )
    )

    with open(project_list_path, "w") as outfile:
      outfile.write(json.dumps(projects_list, indent=4))
    
    print(f'Project named as "{answers["selected_project"]}" has been deleted!')



import json
from os import getcwd
from inquirer import Text, Path, prompt
from .GenericProjectActionsModel import GenericProjectActionsModel

class CreateProject(GenericProjectActionsModel):
  @staticmethod
  def run(project_list_path):
    projects_list = []

    with open(project_list_path, 'r') as openfile:
      projects_list = json.load(openfile)

    questions = [
      Text(
        'name',
        message="What's the project name?",
        validate=lambda answers, current: CreateProject._validate_name(
          current,
          projects_list
        )
      ),
      Path(
        'project_path',
        message="Where is the project located?",
        path_type=Path.DIRECTORY,
        exists=True,
        normalize_to_absolute_path=True,
        default=getcwd()
      ),
    ]

    new_project = prompt(questions)
    if new_project:
      projects_list.append(new_project)

    with open(project_list_path, "w") as outfile:
      outfile.write(json.dumps(projects_list, indent=4))

    if new_project:
      print("New project created!")



from startwork.main import main
from startwork.constants.__version__ import __version__

class TestEndToEnd:
  expected_help_output = f'avaliable options:\n  default: run\n\n  create: create a new project\n\n  delete: delete a project\n\n'

  def test_unkown_option(self, capsys):
    main(["", "--randon_stuff"])
    out, err = capsys.readouterr()
    expeted_help_message = 'Unknown option: --randon_stuff\nTry one of the following:\n'
    assert out == expeted_help_message + self.expected_help_output
    assert err == ''

  def test_help(self, capsys):
    main(["", "--help"])
    out, err = capsys.readouterr()
    assert out == self.expected_help_output
    assert err == ''

  def test_version(self, capsys):
    main(["", "--version"])
    out, err = capsys.readouterr()
    assert out == f'version: {__version__}\n\n'
    assert err == ''



import pytest
import unittest
from unittest.mock import patch

from copy import deepcopy, copy
from pathlib import Path
import json

from startwork.models.CreateProject import CreateProject
from startwork.models.DeleteProject import DeleteProject


mock_values_create = [
  {"name": "sample_name1", "project_path": "/home"},
  {"name": "sample_name2", "project_path": "/home"},
  {"name": "sample_name3", "project_path": "/home"},
]

mock_values_delete = [{
  "selected_project": value_create["name"],
  "delete_confirmed": True,
  "delete_confirmed2": True,
} for value_create in mock_values_create ]

mock_values_delete_unconfirmed = [
  {
    "selected_project": mock_values_create[0]["name"],
    "delete_confirmed": False,
    "delete_confirmed2": False,
  },
  {
    "selected_project": mock_values_create[0]["name"],
    "delete_confirmed": False,
    "delete_confirmed2": True,
  },
  {
    "selected_project": mock_values_create[0]["name"],
    "delete_confirmed": True,
    "delete_confirmed2": False,
  }
]

class TestDeleteProject(unittest.TestCase):
  # CLASS SETUP
  project_list_path = Path(__file__).parent / "projects_list.json"
  mock_values_create = deepcopy(mock_values_create)
  generic_error_exception = Exception("Deletion Aborted!")

  @pytest.fixture(autouse=True)
  def _capsys(self, capsys):
    self.capsys = capsys

  # TESTS
  def test_if_is_function(self):
    assert callable(DeleteProject.run)

  @patch(
    "startwork.models.CreateProject.prompt",
    return_value=mock_values_create[0]
  )
  @patch(
    "startwork.models.DeleteProject.prompt",
    return_value=mock_values_delete[0]
  )
  def test_happy_path(
    self,
    mock_inquirer_prompt_delete,
    mock_inquirer_prompt_create
  ):
    # set json for the test
    for prompt_value in mock_values_create:
      mock_inquirer_prompt_create.return_value["name"] = copy(prompt_value["name"])
      mock_inquirer_prompt_create.return_value["project_path"] = copy(prompt_value["project_path"])

      CreateProject.run(self.project_list_path)

    with open(self.project_list_path, "r") as file:
      result = json.load(file)
      assert result == mock_values_create
    # clean output
    self.capsys.readouterr()

    # do the test it self
    expected_result = mock_values_create
    for prompt_value in mock_values_delete:
      project_name = prompt_value["selected_project"]
      mock_inquirer_prompt_delete.return_value["selected_project"] = copy(project_name)

      DeleteProject.run(self.project_list_path)

      out, err = self.capsys.readouterr()
      assert out == f'Project named as "{project_name}" has been deleted!\n'
      assert err == ""
      # asset comparing array removing each item at time
      with open(self.project_list_path, "r") as file:
        expected_result = list(
          filter(
            lambda mock_value: mock_value["name"] != project_name,
            expected_result
          )
        )
        result = json.load(file)
        assert result == expected_result


  @patch(
    "startwork.models.CreateProject.prompt",
    return_value=mock_values_create[0]
  )
  @patch(
    "startwork.models.DeleteProject.prompt",
    return_value=mock_values_delete_unconfirmed[0]
  )
  def test_delete_unconfirmed(
    self,
    mock_inquirer_prompt_delete,
    mock_inquirer_prompt_create
  ):
    CreateProject.run(self.project_list_path)
    with open(self.project_list_path, "r") as file:
      result = json.load(file)
      assert result == [mock_values_create[0]]
    # clean output
    self.capsys.readouterr()

    for mock_value in mock_values_delete_unconfirmed:
      mock_inquirer_prompt_delete.return_value["delete_confirmed"] = mock_value["delete_confirmed"]
      mock_inquirer_prompt_delete.return_value["delete_confirmed2"] = mock_value["delete_confirmed2"]
      try:
        DeleteProject.run(self.project_list_path)
        assert False
      except Exception as exception:
        assert exception.args == self.generic_error_exception.args

    with open(self.project_list_path, "w") as outfile:
      outfile.write(json.dumps([], indent=4))

    with open(self.project_list_path, "r") as file:
      assert json.load(file) == []



import sys
from pathlib import Path
from subprocess import check_call

from .models.SelectProject import SelectProject
from .models.CreateProject import CreateProject
from .models.DeleteProject import DeleteProject
from .constants.__version__ import __version__

project_list_path = Path(__file__).parent / "projects_list.json"
scripts_path = Path(__file__).parent / "scripts" / "work.sh"

def get_help():
  print("avaliable options:") 
  print("  default: run\n")  
  print("  create: create a new project\n") 
  print("  delete: delete a project\n") 

def main(argv) -> int:
  if len(argv) < 2:
    selected_project = SelectProject(project_list_path)
    start_work_script = [scripts_path, selected_project["project_path"]]
    requirementsPath = Path('requirements.txt')

    print("is file?")
    print(requirementsPath.is_file())
    if requirementsPath.is_file():
      for line in open('requirements.txt'):
        requirement_name = line.split("=")
        if requirement_name == "flask":
          start_work_script.append("flask")
          break

    check_call(start_work_script)
    return


  option = argv[1]

  if option == "create":
    return CreateProject.run(project_list_path)

  if option == "delete":
    return DeleteProject(project_list_path)

  if option == "--version":
    print(f'version: {__version__}\n')  
    return 0

  if option == "--help":
    get_help()
    return 0
  
  print(f'Unknown option: {option}')
  print('Try one of the following:')
  get_help()
  return 1


if __name__ == "__main__":
  main(sys.argv)



from inquirer import errors

class GenericProjectActionsModel:
  @staticmethod
  def _raiseError(reason):
    raise errors.ValidationError('', reason=reason)

  @staticmethod
  def _validate_name(current, projects_list):
    if len(current) < 1:
      GenericProjectActionsModel._raiseError("Project name can't be empty")

    for project in projects_list:
      if project["name"] == current:
        GenericProjectActionsModel._raiseError(f'Name "{current}" alredy in use')

    return True
  
  @staticmethod
  def _validate_selected_project_name(selected_project_name):
    if not selected_project_name:
      raise Exception("Unknown cause error: no project selected")



import json
from inquirer import List, prompt

from .GenericProjectActionsModel import GenericProjectActionsModel

class SelectProject(GenericProjectActionsModel):
  @staticmethod
  def run(project_list_path):
    projects_list = []

    with open(project_list_path, 'r') as openfile:
      projects_list = json.load(openfile)
    
    projects_keys=[project["name"] for project in projects_list]

    questions = [
      List(
        'selected_project',
        message="Selete a project to work on",
        choices=projects_keys,
      )
    ]

    selected_project_name = prompt(questions)["selected_project"]
    SelectProject._validate_selected_project_name(selected_project_name)

    print(f'Starting to work on "{selected_project_name}"')

    selected_project = list(
      filter(
        lambda project: project['name'] == selected_project_name,
        projects_list
      )
    )[0]
    return selected_project



from setuptools import setup, find_packages
from startwork.constants.__version__ import __version__
from os import path
from codecs import open

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = __version__
DESCRIPTION = 'Small CLI to easier start and swap between projects in different stacks.'
LONG_DESCRIPTION = long_description

setup(
    name="startwork",
    version=VERSION,
	entry_points={
        'console_scripts': [
            'work=startwork.main:main'
        ]
    },
    author="JorbFreire",
    author_email="jorbfreire@gmail.com",
    description=DESCRIPTION,
    long_description=long_description,
    include_package_data=True,
    packages=find_packages(),
    install_requires=["inquirer==3.1.2"],
    keywords=['python', 'inquirer', 'cli', 'dev-tools', 'tooling'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
    ]
)



from pprint import pprint

from startwork.models.GenericProjectActionsModel import GenericProjectActionsModel
from inquirer.errors import ValidationError as InquirerValidationError

class TestGenericProjectActionsModel():
  # CLASS SETUP
  prompt_mock_values = [
    {"name": "sample_name1", "project_path": "/home"},
    {"name": "sample_name2", "project_path": "/home"},
    {"name": "sample_name3", "project_path": "/home"},
  ]

  def _get_error(self, error, current=""):
    if error == "empty":
      return InquirerValidationError("", reason="Project name can't be empty")
    if error == "alredy_used" and len(current) > 1:
      return InquirerValidationError("", reason=f'Name "{current}" alredy in use')
    return InquirerValidationError("", reason=error)

  # TESTS
  def test_raiseError(self):
    error_string = "sample_error"
    try:
      GenericProjectActionsModel._raiseError(error_string)
      assert False
    except Exception as exception:
      expected_exception = self._get_error(error_string)
      assert exception.__module__ == expected_exception.__module__
      assert exception.reason == expected_exception.reason
      assert exception.value == expected_exception.value

  def test_validate_name_happy_path(self, capsys):
    try:
      is_valid_name = GenericProjectActionsModel._validate_name(
        "aa",
        self.prompt_mock_values
      )
      out, err = capsys.readouterr()
      assert out == ""
      assert err == ""
      assert is_valid_name
    except Exception:
      assert False  

  def test_validate_name_string_length(self):
    try:
      GenericProjectActionsModel._validate_name(
        "",
        self.prompt_mock_values
      )
      assert False
    except Exception as exception:
      expected_exception = self._get_error("empty")
      assert exception.__module__ == expected_exception.__module__
      assert exception.reason == expected_exception.reason
      assert exception.value == expected_exception.value

  def test_validate_name_alredy_used(self):
    for prompt_value in self.prompt_mock_values:
      try:
        GenericProjectActionsModel._validate_name(
          prompt_value["name"],
          self.prompt_mock_values
        )
        assert False
      except Exception as exception:
        expected_exception = self._get_error("alredy_used", prompt_value["name"])
        assert exception.__module__ == expected_exception.__module__
        assert exception.reason == expected_exception.reason
        assert exception.value == expected_exception.value



import pytest
import unittest
from unittest.mock import patch

from copy import deepcopy, copy
from pathlib import Path
import json

from startwork.models.CreateProject import CreateProject

prompt_mock_values = [
  {"name": "sample_name1", "project_path": "/home"},
  {"name": "sample_name2", "project_path": "/home"},
  {"name": "sample_name3", "project_path": "/home"},
]

class TestCreateProject(unittest.TestCase):
  # CLASS SETUP
  project_list_path = Path(__file__).parent / "projects_list.json"
  prompt_mock_values = deepcopy(prompt_mock_values)

  @pytest.fixture(autouse=True)
  def _capsys(self, capsys):
    self.capsys = capsys

  # TESTS
  def test_if_is_function(self):
    assert callable(CreateProject.run)

  @patch(
    "startwork.models.CreateProject.prompt",
    return_value=prompt_mock_values[0]
  )
  def test_happy_path(self, mock_inquirer_prompt):
    for prompt_value in prompt_mock_values:
      mock_inquirer_prompt.return_value["name"] = copy(prompt_value["name"])
      mock_inquirer_prompt.return_value["project_path"] = copy(prompt_value["project_path"])

      CreateProject.run(self.project_list_path)
      out, err = self.capsys.readouterr()
      assert out == "New project created!\n"
      assert err == ""

    with open(self.project_list_path, "r") as file:
      assert json.load(file) == prompt_mock_values

    # clean json for the next test 
    with open(self.project_list_path, "w") as outfile:
      outfile.write(json.dumps([], indent=4))

    with open(self.project_list_path, "r") as file:
      assert json.load(file) == []
